/*	NSValue.h
	Copyright 1994 NeXT, Inc. All rights reserved.

	Use NSValues to put random C-types into collections.
	Use NSNumbers to put numbers into collections.

*/
#import <foundation/NSObject.h>
#import <foundation/NSString.h>

/* An abstract class for wrapping any C-type with an object. Sample use: To put an NSRange into an NSArray, you would [myArray insertObject:[NSValue value:&range withObjCType:@encode(NSRange)] atIndex:n]; to get it back out, [[myArray objectAtIndex:n] getValue:&range].
*/
@interface NSValue : NSObject <NSCopying>

+ (NSValue *)value:(const void *)value withObjCType:(const char *)type;	/* Creation method */

- (void)getValue:(void *)value;	/* Value copied from the object to caller's variable... */
- (const char *)objCType;	/* Standard objC encoding string */

@end


@interface NSValue (NSValueOtherTypes)

+ (NSValue *)valueWithNonretainedObject:anObject;	/* Get objects into collections without retain */
- nonretainedObjectValue;

+ (NSValue *)valueWithPointer:(void *)pointer;	/* Get pointers into collections */
- (void *)pointerValue;

@end


/* An abstract class for wrapping C number types with an object. NSNumbers have the added feature that they can do type-coercion. The objCType method in an NSNumber can only be one of c, C, s, S, i, I, l, L, q, Q, f, or d. 
*/
@interface NSNumber : NSValue

/* These methods return the number in the requested type. These methods will do type-coercion as defined by C. In certain cases the results may be undefined; for instance, trying to take the unsigned integer value of a negative number, or trying to get the short value of an float which is greater than the maximum short.
*/
- (char)charValue;
- (unsigned char)unsignedCharValue;
- (short)shortValue;
- (unsigned short)unsignedShortValue;
- (int)intValue;
- (unsigned int)unsignedIntValue;
- (long)longValue;
- (unsigned long)unsignedLongValue;
- (long long)longLongValue;
- (unsigned long long)unsignedLongLongValue;
- (float)floatValue;
- (double)doubleValue;
- (BOOL)boolValue;
- (NSString *)stringValue;

/* Compare the numbers exactly like C would.
*/
- (NSComparisonResult)compare:(NSNumber *)otherNumber;
@end

/* Creation methods for built-in subclasses of NSNumber
*/
@interface NSNumber (NSNumberCreation)
+ (NSNumber *)numberWithChar:(char)value;
+ (NSNumber *)numberWithUnsignedChar:(unsigned char)value;
+ (NSNumber *)numberWithShort:(short)value;
+ (NSNumber *)numberWithUnsignedShort:(unsigned short)value;
+ (NSNumber *)numberWithInt:(int)value;
+ (NSNumber *)numberWithUnsignedInt:(unsigned int)value;
+ (NSNumber *)numberWithLong:(long)value;
+ (NSNumber *)numberWithUnsignedLong:(unsigned long)value;
+ (NSNumber *)numberWithLongLong:(long long)value;
+ (NSNumber *)numberWithUnsignedLongLong:(unsigned long long)value;
+ (NSNumber *)numberWithFloat:(float)value;
+ (NSNumber *)numberWithDouble:(double)value;
+ (NSNumber *)numberWithBool:(BOOL)value;

- initWithChar:(char)value;
- initWithUnsignedChar:(unsigned char)value;
- initWithShort:(short)value;
- initWithUnsignedShort:(unsigned short)value;
- initWithInt:(int)value;
- initWithUnsignedInt:(unsigned int)value;
- initWithLong:(long)value;
- initWithUnsignedLong:(unsigned long)value;
- initWithLongLong:(long long)value;
- initWithUnsignedLongLong:(unsigned long long)value;
- initWithFloat:(float)value;
- initWithDouble:(double)value;
- initWithBool:(BOOL)value;
@end

